==> scanbot_sdk-2.13.2-alpha01.tar.gz: ios/Classes/Classical Component/DocumentScanner.swift <==
//
//  DocumentScanner.swift
//  scanbot_sdk
//
//  Created by Serhii Chaban on 27.06.22.
//

import Foundation
import ScanbotSDK

class DocumentScanner: NSObject, ScannerDelegate, SBSDKDocumentScannerViewControllerDelegate, CallErrorHandler {
    
    private var viewController: SBSDKDocumentScannerViewController?
    private var cameraMethodChannel: FlutterMethodChannel?
    private let detectorChannel: FlutterMethodChannel?
    private let finderChannel: FlutterMethodChannel?
    private var shouldPauseDetectionOnResultReturn: Bool = false
    private var configuration: [String: Any?]
    internal var shouldRunDetection: Bool = true
    internal var showPolygonInManualMode: Bool = false
    private var detectDocumentAfterSnap: Bool = false
    private var customSnapButton: UIButton?
    private var originalImageSizeLimit: CGSize? = CGSize.zero
    
    init(
        cameraMethodChannel: FlutterMethodChannel?,
        detectorChannel: FlutterMethodChannel?,
        finderChannel: FlutterMethodChannel?,
        configuration: [String: Any?]
    ) {
        self.cameraMethodChannel = cameraMethodChannel
        self.detectorChannel = detectorChannel
        self.finderChannel = finderChannel
        self.configuration = configuration
    }
    
    func createNativeView(view: UIView) {
        if !ScanbotSDK.isLicenseValid() {
            DispatchQueue.main.async { [weak self] in
                guard let callHandler = self?.detectorChannel else {
                    fatalError("callHandler cannot be nil")
                }
                LicenseHandler.sendLicenseErrorToDetector(callHandler)
            }
            return
        }
        customSnapButton = UIButton(frame: CGRect(x: 100, y: 100, width: 100, height: 50))
        
        viewController = SBSDKDocumentScannerViewController()
        viewController?.delegate = self
        viewController?.customSnapButton = customSnapButton
        viewController?.attachFlutterView(view)
        _ = viewController?.view
        
        setDocumentScannerConfiguration(configuration: configuration)
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnCameraOpenMethod, arguments: "")
    }
    
    func attach(view: UIView) {
        createNativeView(view: view)
        detectorChannel?.setMethodCallHandler({ (call, result) in
            self.handleDetectorCall(call: call, result: result)
        })
    }
    
    func setFlash(enabled: Bool) {
        viewController?.isFlashLightEnabled = enabled
    }
    
    func isFlashAvailable() -> Bool {
        return viewController?.isFlashLightAvailable ?? false
    }
    
    func documentScannerViewController(_ controller: SBSDKDocumentScannerViewController,
                                       didSnapDocumentImage documentImage: UIImage,
                                       on originalImage: UIImage,
                                       with result: SBSDKDocumentDetectorResult,
                                       autoSnapped: Bool) {
        shouldPauseDetectionOnResultReturn = true
       
        var originalImage = originalImage
        
        if let imageSizeLimit = self.originalImageSizeLimit, imageSizeLimit != CGSize.zero {
            originalImage = originalImage.sbsdk_limited(to: imageSizeLimit) ?? originalImage
        }
       
        let storage = SBSDKUIPageFileStorage.default()
        guard let uuid = storage.add(originalImage) else { return }

        let page = SBSDKUIPage.init(pageFileID: uuid, polygon: SBSDKPolygon())
        
        if let documentImageSizeLimit = viewController?.documentImageSizeLimit, documentImageSizeLimit != CGSize.zero {
            page.documentImageSizeLimit = documentImageSizeLimit
        }
        
        if detectDocumentAfterSnap {
            do{
                try storage.setImage(documentImage,
                                     forExistingPageFileID: uuid,
                                     pageFileType: SBSDKUIPageFileType.documentUnfiltered)
                try storage.setImage(documentImage,
                                     forExistingPageFileID: uuid,
                                     pageFileType: SBSDKUIPageFileType.document)
                page.polygon = result.polygon
            }
            catch {
                print("ScanbotSdk","Error creating document images", "Can't set document image for page object")
            }
   
        }
        
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnShowHeavyProcessing, arguments: false)
        sendSnappedPage(page) { [weak self] in
            self?.shouldPauseDetectionOnResultReturn = false
        }
    }
    
    private func sendSnappedPage(_ page: SBSDKUIPage,
                                    completion: @escaping () -> Void) {
        DispatchQueue.main.async { [weak self] in
            guard let callHandler = self?.cameraMethodChannel else {
                fatalError("callHandler cannot be nil")
            }
            callHandler.invokeMethod(MethodChannelConstants.kOnPictureTaken, arguments: page.encodedString){ _ in
                completion()
            }
        }
    }
    
    func documentScannerViewControllerShouldDetectDocument(_ controller: SBSDKDocumentScannerViewController) -> Bool {
        return shouldRunDetection && !shouldPauseDetectionOnResultReturn
    }
    
    func documentScannerViewControllerWillSnapImage(_ controller: SBSDKDocumentScannerViewController) {
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnShowHeavyProcessing, arguments: true)
    }
    
    func documentScannerViewControllerDidFailSnappingImage(_ controller: SBSDKDocumentScannerViewController,
                                                           withError error: Error?) {
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnShowHeavyProcessing, arguments: false)
    }
    
    func documentScannerController(_ controller: SBSDKDocumentScannerViewController,
                                   didSampleVideoFrame videoFrameImage: UIImage,
                                   detectionResult result: SBSDKDocumentDetectorResult) {
        sendDetectedResult(result) { [weak self] in
            self?.shouldPauseDetectionOnResultReturn = false
        }
    }
    
    private func sendDetectedResult(_ result: SBSDKDocumentDetectorResult,
                                    completion: @escaping () -> Void) {
  
        DispatchQueue.main.async { [weak self] in
            guard let callHandler = self?.detectorChannel else {
                fatalError("callHandler cannot be nil")
            }
            callHandler.invokeMethod(MethodChannelConstants.kOnContourDetected, arguments: result.encodedString){ _ in
                completion()
            }
        }
    }
    
    func setDocumentScannerConfiguration(configuration: [String: Any?]?) {
        let documentScannerConfiguration = DocumentScannerConfiguration(configuration)
        applyPolygonConfiguration(documentScannerConfiguration.polygonConfiguration)
        
        if let flashEnabled = documentScannerConfiguration.flashEnabled {
            viewController?.isFlashLightEnabled = flashEnabled
        }
        
        if let zoomConfiguration = documentScannerConfiguration.zoomConfiguration {
            viewController?.zoomConfiguration = zoomConfiguration
        }
        
        applyScannerConfiguration(documentScannerConfiguration.scannerConfiguration)
        
    }
    
    private func applyScannerConfiguration(_ configuration: DocumentScannerConfiguration.ScannerConfiguration?) {
        if let scannerConfiguration = configuration {
            
            viewController?.statusTextConfiguration.textHintBadAngles = scannerConfiguration.textHintBadAngles
            viewController?.statusTextConfiguration.textHintTooDark = scannerConfiguration.textHintTooDark
            viewController?.statusTextConfiguration.textHintTooNoisy = scannerConfiguration.textHintTooNoisy
            viewController?.statusTextConfiguration.textHintTooSmall = scannerConfiguration.textHintTooSmall
            viewController?.statusTextConfiguration.textHintOffCenter = scannerConfiguration.textHintOffCenter
            viewController?.statusTextConfiguration.textHintNothingDetected = scannerConfiguration.textHintNothingDetected
            viewController?.statusTextConfiguration.textHintOKManualScan = scannerConfiguration.textHintOKManualScan
            viewController?.statusTextConfiguration.textHintOKAutoShutter = scannerConfiguration.textHintOKAutoShutter
            viewController?.statusTextConfiguration.textHintEnergySavingActive = scannerConfiguration.textHintEnergySavingActive
            viewController?.statusTextConfiguration.textHintBadAspectRatio = scannerConfiguration.textHintBadAspectRatio
            viewController?.hideSnapButton = scannerConfiguration.hideSnapButton
            
            if let acceptedSizeScore = scannerConfiguration.acceptedSizeScore {
                viewController?.acceptedSizeScore = acceptedSizeScore
            }
            
            if let acceptedAngleScore = scannerConfiguration.acceptedAngleScore {
                viewController?.acceptedAngleScore = acceptedAngleScore
            }
            
            if let ignoreBadAspectRatio = scannerConfiguration.ignoreBadAspectRatio {
                viewController?.ignoreBadAspectRatio = ignoreBadAspectRatio
            }
            
            if let autoSnappingSensitivity = scannerConfiguration.autoSnappingSensitivity {
                viewController?.autoSnappingSensitivity = autoSnappingSensitivity
            }
            
            if let detectDocumentAfterSnap = scannerConfiguration.detectDocumentAfterSnap {
                self.detectDocumentAfterSnap = detectDocumentAfterSnap
            }
            
            if let documentImageSizeLimit = scannerConfiguration.documentImageSizeLimit {
                viewController?.documentImageSizeLimit = documentImageSizeLimit
            }
            
            if let originalImageSizeLimit = scannerConfiguration.originalImageSizeLimit {
                self.originalImageSizeLimit = originalImageSizeLimit
            }
            
            if let autoSnapEnabled = scannerConfiguration.autoSnapEnabled {
                if(autoSnapEnabled){
                    viewController?.autoSnappingMode = SBSDKAutosnappingMode.enabled
                } else {
                    viewController?.autoSnappingMode = SBSDKAutosnappingMode.disabled
                }
                viewController?.suppressPolygonLayer = !autoSnapEnabled && showPolygonInManualMode
                viewController?.hidePolygonLayer = !autoSnapEnabled && showPolygonInManualMode
            }
            
            if let requiredAspectRatios = scannerConfiguration.requiredAspectRatios {
                viewController?.requiredAspectRatios = requiredAspectRatios
            }
        }
    }
    
    private func applyPolygonConfiguration(_ configuration: DocumentScannerConfiguration.PolygonConfiguration?) {
        if let polygonConfiguration = configuration {
            
            if let showPolygonInManualMode = polygonConfiguration.showPolygonInManualMode {
                self.showPolygonInManualMode = showPolygonInManualMode
            }
            
            if let autoSnapProgressEnabled = polygonConfiguration.polygonAutoSnapProgressEnabled {
                viewController?.polygonAutoSnapProgressEnabled = autoSnapProgressEnabled
            }
            
            if let polygonLineWidth = polygonConfiguration.polygonLineWidth {
                viewController?.polygonLineWidth = polygonLineWidth
            }
            
            if let polygonAutoSnapProgressLineWidth = polygonConfiguration.polygonAutoSnapProgressLineWidth {
                viewController?.polygonAutoSnapProgressLineWidth = polygonAutoSnapProgressLineWidth
            }
            
            if let polygonCornerRadius = polygonConfiguration.polygonCornerRadius {
                viewController?.polygonCornerRadius = polygonCornerRadius
            }
            
            if let polygonFillColorAccepted = polygonConfiguration.polygonFillColorAccepted {
                viewController?.polygonFillColorAccepted = polygonFillColorAccepted
            }
            
            if let polygonFillColorRejected = polygonConfiguration.polygonFillColorRejected {
                viewController?.polygonFillColorRejected = polygonFillColorRejected
            }
            
            if let polygonLineColorAccepted = polygonConfiguration.polygonLineColorAccepted {
                viewController?.polygonLineColorAccepted = polygonLineColorAccepted
            }
            
            if let polygonLineColorRejected = polygonConfiguration.polygonLineColorRejected {
                viewController?.polygonLineColorRejected = polygonLineColorRejected
            }
            
            if let polygonAutoSnapProgressColor = polygonConfiguration.polygonAutoSnapProgressColor {
                viewController?.polygonAutoSnapProgressColor = polygonAutoSnapProgressColor
            }
        }
    }
    
    func stopPreview() {
        viewController?.freezeCamera()
    }
    
    func resumePreview() {
        viewController?.unfreezeCamera()
    }
    
    func handleDetectorCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as? [String: Any]
        switch call.method {
        case "enable": shouldRunDetection = true
        case "disable": shouldRunDetection = false
        case "makeSnap": customSnapButton?.sendActions(for: .touchUpInside)
        case "setAutoSnappingEnabled": do {
            guard let autoSnapEnabled = arguments?["enabled"] as? Bool else {
                return
            }
            viewController?.suppressPolygonLayer = !autoSnapEnabled && showPolygonInManualMode
            viewController?.hidePolygonLayer = !autoSnapEnabled && showPolygonInManualMode
            self.viewController?.autoSnappingMode = autoSnapEnabled ? .enabled : .disabled
        }
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        returnSuccess(result: result) // always return some result to the flutter
    }
    
    func returnSuccess(result: @escaping FlutterResult) {
        result("success")
    }
}

==> scanbot_sdk-2.13.3-alpha01.tar.gz: ios/Classes/Classical Component/DocumentScanner.swift <==
//
//  DocumentScanner.swift
//  scanbot_sdk
//
//  Created by Serhii Chaban on 27.06.22.
//

import Foundation
import ScanbotSDK

class DocumentScanner: NSObject, ScannerDelegate, SBSDKDocumentScannerViewControllerDelegate, CallErrorHandler {
    
    private var viewController: SBSDKDocumentScannerViewController?
    private var cameraMethodChannel: FlutterMethodChannel?
    private let detectorChannel: FlutterMethodChannel?
    private let finderChannel: FlutterMethodChannel?
    private var shouldPauseDetectionOnResultReturn: Bool = false
    private var configuration: [String: Any?]
    internal var shouldRunDetection: Bool = true
    internal var showPolygonInManualMode: Bool = false
    private var detectDocumentAfterSnap: Bool = false
    private var customSnapButton: UIButton?
    private var originalImageSizeLimit: CGSize? = CGSize.zero
    
    init(
        cameraMethodChannel: FlutterMethodChannel?,
        detectorChannel: FlutterMethodChannel?,
        finderChannel: FlutterMethodChannel?,
        configuration: [String: Any?]
    ) {
        self.cameraMethodChannel = cameraMethodChannel
        self.detectorChannel = detectorChannel
        self.finderChannel = finderChannel
        self.configuration = configuration
    }
    
    func createNativeView(view: UIView) {
        if !ScanbotSDK.isLicenseValid() {
            DispatchQueue.main.async { [weak self] in
                guard let callHandler = self?.detectorChannel else {
                    fatalError("callHandler cannot be nil")
                }
                LicenseHandler.sendLicenseErrorToDetector(callHandler)
            }
            return
        }
        customSnapButton = UIButton(frame: CGRect(x: 100, y: 100, width: 100, height: 50))
        
        viewController = SBSDKDocumentScannerViewController()
        viewController?.delegate = self
        viewController?.customSnapButton = customSnapButton
        viewController?.attachFlutterView(view)
        _ = viewController?.view
        
        setDocumentScannerConfiguration(configuration: configuration)
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnCameraOpenMethod, arguments: "")
    }
    
    func attach(view: UIView) {
        createNativeView(view: view)
        detectorChannel?.setMethodCallHandler({ (call, result) in
            self.handleDetectorCall(call: call, result: result)
        })
    }
    
    func setFlash(enabled: Bool) {
        viewController?.isFlashLightEnabled = enabled
    }
    
    func isFlashAvailable() -> Bool {
        return viewController?.isFlashLightAvailable ?? false
    }
    
    func documentScannerViewController(_ controller: SBSDKDocumentScannerViewController,
                                       didSnapDocumentImage documentImage: UIImage,
                                       on originalImage: UIImage,
                                       with result: SBSDKDocumentDetectorResult,
                                       autoSnapped: Bool) {
        shouldPauseDetectionOnResultReturn = true
       
        var originalImage = originalImage
        
        if let imageSizeLimit = self.originalImageSizeLimit, imageSizeLimit != CGSize.zero {
            originalImage = originalImage.sbsdk_limited(to: imageSizeLimit) ?? originalImage
        }
       
        let storage = SBSDKUIPageFileStorage.default()
        guard let uuid = storage.add(originalImage) else { return }

        let page = SBSDKUIPage.init(pageFileID: uuid, polygon: SBSDKPolygon())
        
        if let documentImageSizeLimit = viewController?.documentImageSizeLimit, documentImageSizeLimit != CGSize.zero {
            page.documentImageSizeLimit = documentImageSizeLimit
        }
        
        if detectDocumentAfterSnap {
            do{
                try storage.setImage(documentImage,
                                     forExistingPageFileID: uuid,
                                     pageFileType: SBSDKUIPageFileType.documentUnfiltered)
                try storage.setImage(documentImage,
                                     forExistingPageFileID: uuid,
                                     pageFileType: SBSDKUIPageFileType.document)
                page.polygon = result.polygon
            }
            catch {
                print("ScanbotSdk","Error creating document images", "Can't set document image for page object")
            }
   
        }
        
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnShowHeavyProcessing, arguments: false)
        sendSnappedPage(page) { [weak self] in
            self?.shouldPauseDetectionOnResultReturn = false
        }
    }
    
    private func sendSnappedPage(_ page: SBSDKUIPage,
                                    completion: @escaping () -> Void) {
        DispatchQueue.main.async { [weak self] in
            guard let callHandler = self?.cameraMethodChannel else {
                fatalError("callHandler cannot be nil")
            }
            callHandler.invokeMethod(MethodChannelConstants.kOnPictureTaken, arguments: page.encodedString){ _ in
                completion()
            }
        }
    }
    
    func documentScannerViewControllerShouldDetectDocument(_ controller: SBSDKDocumentScannerViewController) -> Bool {
        return shouldRunDetection && !shouldPauseDetectionOnResultReturn
    }
    
    func documentScannerViewControllerWillSnapImage(_ controller: SBSDKDocumentScannerViewController) {
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnShowHeavyProcessing, arguments: true)
    }
    
    func documentScannerViewControllerDidFailSnappingImage(_ controller: SBSDKDocumentScannerViewController,
                                                           withError error: Error?) {
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnShowHeavyProcessing, arguments: false)
    }
    
    func documentScannerController(_ controller: SBSDKDocumentScannerViewController,
                                   didSampleVideoFrame videoFrameImage: UIImage,
                                   detectionResult result: SBSDKDocumentDetectorResult) {
        sendDetectedResult(result) { [weak self] in
            self?.shouldPauseDetectionOnResultReturn = false
        }
    }
    
    private func sendDetectedResult(_ result: SBSDKDocumentDetectorResult,
                                    completion: @escaping () -> Void) {
  
        DispatchQueue.main.async { [weak self] in
            guard let callHandler = self?.detectorChannel else {
                fatalError("callHandler cannot be nil")
            }
            callHandler.invokeMethod(MethodChannelConstants.kOnContourDetected, arguments: result.encodedString){ _ in
                completion()
            }
        }
    }
    
    func setDocumentScannerConfiguration(configuration: [String: Any?]?) {
        let documentScannerConfiguration = DocumentScannerConfiguration(configuration)
        applyPolygonConfiguration(documentScannerConfiguration.polygonConfiguration)
        
        if let flashEnabled = documentScannerConfiguration.flashEnabled {
            viewController?.isFlashLightEnabled = flashEnabled
        }
        
        if let zoomConfiguration = documentScannerConfiguration.zoomConfiguration {
            viewController?.zoomConfiguration = zoomConfiguration
        }
        
        applyScannerConfiguration(documentScannerConfiguration.scannerConfiguration)
        
    }
    
    private func applyScannerConfiguration(_ configuration: DocumentScannerConfiguration.ScannerConfiguration?) {
        if let scannerConfiguration = configuration {
            
            viewController?.statusTextConfiguration.textHintBadAngles = scannerConfiguration.textHintBadAngles
            viewController?.statusTextConfiguration.textHintTooDark = scannerConfiguration.textHintTooDark
            viewController?.statusTextConfiguration.textHintTooNoisy = scannerConfiguration.textHintTooNoisy
            viewController?.statusTextConfiguration.textHintTooSmall = scannerConfiguration.textHintTooSmall
            viewController?.statusTextConfiguration.textHintOffCenter = scannerConfiguration.textHintOffCenter
            viewController?.statusTextConfiguration.textHintNothingDetected = scannerConfiguration.textHintNothingDetected
            viewController?.statusTextConfiguration.textHintOKManualScan = scannerConfiguration.textHintOKManualScan
            viewController?.statusTextConfiguration.textHintOKAutoShutter = scannerConfiguration.textHintOKAutoShutter
            viewController?.statusTextConfiguration.textHintEnergySavingActive = scannerConfiguration.textHintEnergySavingActive
            viewController?.statusTextConfiguration.textHintBadAspectRatio = scannerConfiguration.textHintBadAspectRatio
            viewController?.hideSnapButton = scannerConfiguration.hideSnapButton
            
            if let acceptedSizeScore = scannerConfiguration.acceptedSizeScore {
                viewController?.acceptedSizeScore = acceptedSizeScore
            }
            
            if let acceptedAngleScore = scannerConfiguration.acceptedAngleScore {
                viewController?.acceptedAngleScore = acceptedAngleScore
            }
            
            if let ignoreBadAspectRatio = scannerConfiguration.ignoreBadAspectRatio {
                viewController?.ignoreBadAspectRatio = ignoreBadAspectRatio
            }
            
            if let autoSnappingSensitivity = scannerConfiguration.autoSnappingSensitivity {
                viewController?.autoSnappingSensitivity = autoSnappingSensitivity
            }
            
            if let detectDocumentAfterSnap = scannerConfiguration.detectDocumentAfterSnap {
                self.detectDocumentAfterSnap = detectDocumentAfterSnap
            }
            
            if let documentImageSizeLimit = scannerConfiguration.documentImageSizeLimit {
                viewController?.documentImageSizeLimit = documentImageSizeLimit
            }
            
            if let originalImageSizeLimit = scannerConfiguration.originalImageSizeLimit {
                self.originalImageSizeLimit = originalImageSizeLimit
            }
            
            if let autoSnapEnabled = scannerConfiguration.autoSnapEnabled {
                if(autoSnapEnabled){
                    viewController?.autoSnappingMode = SBSDKAutosnappingMode.enabled
                    viewController?.suppressPolygonLayer = false
                    viewController?.hidePolygonLayer = false
                } else {
                    viewController?.autoSnappingMode = SBSDKAutosnappingMode.disabled
                    viewController?.suppressPolygonLayer = !showPolygonInManualMode
                    viewController?.hidePolygonLayer = !showPolygonInManualMode
                }
            }
            
            if let requiredAspectRatios = scannerConfiguration.requiredAspectRatios {
                viewController?.requiredAspectRatios = requiredAspectRatios
            }
        }
    }
    
    private func applyPolygonConfiguration(_ configuration: DocumentScannerConfiguration.PolygonConfiguration?) {
        if let polygonConfiguration = configuration {
            
            if let showPolygonInManualMode = polygonConfiguration.showPolygonInManualMode {
                self.showPolygonInManualMode = showPolygonInManualMode
            }
            
            if let autoSnapProgressEnabled = polygonConfiguration.polygonAutoSnapProgressEnabled {
                viewController?.polygonAutoSnapProgressEnabled = autoSnapProgressEnabled
            }
            
            if let polygonLineWidth = polygonConfiguration.polygonLineWidth {
                viewController?.polygonLineWidth = polygonLineWidth
            }
            
            if let polygonAutoSnapProgressLineWidth = polygonConfiguration.polygonAutoSnapProgressLineWidth {
                viewController?.polygonAutoSnapProgressLineWidth = polygonAutoSnapProgressLineWidth
            }
            
            if let polygonCornerRadius = polygonConfiguration.polygonCornerRadius {
                viewController?.polygonCornerRadius = polygonCornerRadius
            }
            
            if let polygonFillColorAccepted = polygonConfiguration.polygonFillColorAccepted {
                viewController?.polygonFillColorAccepted = polygonFillColorAccepted
            }
            
            if let polygonFillColorRejected = polygonConfiguration.polygonFillColorRejected {
                viewController?.polygonFillColorRejected = polygonFillColorRejected
            }
            
            if let polygonLineColorAccepted = polygonConfiguration.polygonLineColorAccepted {
                viewController?.polygonLineColorAccepted = polygonLineColorAccepted
            }
            
            if let polygonLineColorRejected = polygonConfiguration.polygonLineColorRejected {
                viewController?.polygonLineColorRejected = polygonLineColorRejected
            }
            
            if let polygonAutoSnapProgressColor = polygonConfiguration.polygonAutoSnapProgressColor {
                viewController?.polygonAutoSnapProgressColor = polygonAutoSnapProgressColor
            }
        }
    }
    
    func stopPreview() {
        viewController?.freezeCamera()
    }
    
    func resumePreview() {
        viewController?.unfreezeCamera()
    }
    
    func handleDetectorCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as? [String: Any]
        switch call.method {
        case "enable": shouldRunDetection = true
        case "disable": shouldRunDetection = false
        case "makeSnap": customSnapButton?.sendActions(for: .touchUpInside)
        case "setAutoSnappingEnabled": do {
            guard let autoSnapEnabled = arguments?["enabled"] as? Bool else {
                return
            }
            if(autoSnapEnabled){
                viewController?.autoSnappingMode = SBSDKAutosnappingMode.enabled
                viewController?.suppressPolygonLayer = false
                viewController?.hidePolygonLayer = false
            } else {
                viewController?.autoSnappingMode = SBSDKAutosnappingMode.disabled
                viewController?.suppressPolygonLayer = !showPolygonInManualMode
                viewController?.hidePolygonLayer = !showPolygonInManualMode
            }
        }
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        returnSuccess(result: result) // always return some result to the flutter
    }
    
    func returnSuccess(result: @escaping FlutterResult) {
        result("success")
    }
}
